/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
     Minimal working examples to build upon and 
	 modify to gain an understanding of the API

	 Extensions to the Vulkan Graphics Coffee Break Series
	   
/*
/****************************************************************/

/*
	Simple geometry shader - draws `normal' vector for each 
	triangle and wireframe view of the geometry (triangle edges)
*/

#version 420

layout(triangles) in;
layout(line_strip, max_vertices=32) out;

in VertexData 
{
  vec4  Position;
  vec4  Color;
  vec2  UV;
} VertexIn[];

out VertexData 
{
  vec4  Position;
  vec4  Color;
  vec2  UV;
} VertexOut;


layout (binding = 0) uniform UBO 
{
  mat4 inProjectionMatrix;
  mat4 inViewMatrix;
  mat4 inModelMatrix;
};


void main()
{
	mat4 mvp = inModelMatrix * inViewMatrix * inProjectionMatrix;

	// Draw input triangle as a wireframe

	for(int i=0; i<gl_in.length(); i++)
	{
		vec3 Pa = gl_in[(i+0)%3].gl_Position.xyz;
		vec3 Pb = gl_in[(i+1)%3].gl_Position.xyz;
    
		// Wireframe original triangle
		gl_Position = vec4(Pa, 1.0) * mvp;
		VertexOut.Position = gl_Position;
		VertexOut.UV       = vec2(0,0);
		VertexOut.Color    = vec4(1, 1, 0, 1);
		EmitVertex();
    
		gl_Position = vec4(Pb, 1.0) * mvp;
		VertexOut.Position = gl_Position;
		VertexOut.UV       = vec2(0,0);
		VertexOut.Color    = vec4(1, 1, 0, 1);
		EmitVertex();
		EndPrimitive();
	}



  
  // Draw normal for each face (in middle of the triangle)

  vec3 P0 = gl_in[0].gl_Position.xyz;
  vec3 P1 = gl_in[1].gl_Position.xyz;
  vec3 P2 = gl_in[2].gl_Position.xyz;
  
  
  // If we want to calculate the normal
  vec3 V0 = P0 - P1;
  vec3 V1 = P2 - P1;
  
  vec3 N = cross(V1, V0);
  N = normalize(N);
 
  /*
  // If the normal is passed in
  vec3 N = VertexIn[0].Normal.xyz;
  */

  // Center of the triangle
  vec3 P = (P0+P1+P2) / 3.0;
  
  gl_Position = vec4(P, 1.0) * mvp;
  VertexOut.Position = gl_Position;
  VertexOut.UV       = vec2(0,0);
  VertexOut.Color    = vec4(1, 0, 0, 1);
  EmitVertex();
  
  gl_Position = vec4(P + N * 0.3, 1.0) * mvp;
  VertexOut.Position = gl_Position;
  VertexOut.UV       = vec2(0,0);
  VertexOut.Color    = vec4(1, 0, 0, 1);
  EmitVertex();
  EndPrimitive();

}// End main(..)